@@ -16,7 +16,7 @@ use Net::Config;
 @ISA = qw(Exporter);
 @EXPORT_OK = qw(hostname hostdomain hostfqdn domainname);
 
-$VERSION = "2.13"; # $Id: //depot/libnet/Net/Domain.pm#10 $
+$VERSION = "2.16"; # $Id: //depot/libnet/Net/Domain.pm#17 $
 
 my($host,$domain,$fqdn) = (undef,undef,undef);
 
@@ -101,7 +101,7 @@ sub _hostname {
     	    $host = "";
 	};
     }
- 
+
     # remove garbage 
     $host =~ s/[\0\r\n]+//go;
     $host =~ s/(\A\.+|\.+\Z)//go;
@@ -165,7 +165,7 @@ sub _hostdomain {
         };
 
 	chop($dom = `domainname 2>/dev/null`)
-		unless(defined $dom);
+		unless(defined $dom || $^O =~ /^(?:cygwin|MSWin32)/);
 
 	if(defined $dom) {
 	    my @h = ();
@@ -199,7 +199,7 @@ sub _hostdomain {
 
     # Look for environment variable
 
-    $domain ||= $ENV{LOCALDOMAIN} ||= $ENV{DOMAIN} || undef;
+    $domain ||= $ENV{LOCALDOMAIN} || $ENV{DOMAIN};
 
     if(defined $domain) {
     	$domain =~ s/[\r\n\0]+//g;
@@ -224,19 +224,20 @@ sub domainname {
     # eleminate DNS lookups
 
     return $fqdn = $host . "." . $domain
-	if($host !~ /\./ && $domain =~ /\./);
+	if(defined $host and defined $domain
+		and $host !~ /\./ and $domain =~ /\./);
 
     # For hosts that have no name, just an IP address
-    return $fqdn = $host if $host =~ /^\d+(\.\d+){3}$/;
+    return $fqdn = $host if defined $host and $host =~ /^\d+(\.\d+){3}$/;
 
-    my @host   = split(/\./, $host);
-    my @domain = split(/\./, $domain);
+    my @host   = defined $host   ? split(/\./, $host)   : ('localhost');
+    my @domain = defined $domain ? split(/\./, $domain) : ('');
     my @fqdn   = ();
 
     # Determine from @host & @domain the FQDN
 
     my @d = @domain;
- 
+
 LOOP:
     while(1) {
     	my @h = @host;
@@ -328,4 +329,8 @@ Copyright (c) 1995-1998 Graham Barr. All rights reserved.
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
+=for html <hr>
+
+I<$Id: //depot/libnet/Net/Domain.pm#17 $>
+
 =cut
